/*-----------------------------------------------------------------------------
 * Name:    SerialRS232
 * Purpose: Low Level Serial Definitions
 *-----------------------------------------------------------------------------
 * 
 * 24.05.13 Create By Christian Didriksson
 * 02.11.13 Modified to support REV B Andreas d'Erasmo
 *
 * Copyright (c) 2008 FMC Technolgies. All rights reserved.
 *----------------------------------------------------------------------------*/

#include <MK10F12.h>              /* MK10F12 Definitions          */
#include "SerialDebug.h"
#include "Charq.h"

#ifdef __DBG_ITM
volatile int32_t ITM_RxBuffer;          /* CMSIS Debug Input                  */
#endif

static UART_Type* pUart = 0;
static uint32_t numInts = 0;
static uint32_t numRxInts = 0;
static uint32_t numRxDropped = 0;
static uint32_t numTxInts = 0;
static uint32_t numTxDropped = 0;

static uint8_t  txStarted = 0;

#define RX_QUEUE_SIZE 256
#define TX_QUEUE_SIZE 256

/* The input queue */
CHARQ_STRUCT rxQueue;
   
/* The output queue */
CHARQ_STRUCT txQueue;
#ifdef DEBUG_UART1
void UART1_RX_TX_IRQHandler (void)
#else
void UART2_RX_TX_IRQHandler (void)
#endif
{
  
  
    volatile uint8_t stat = 0; 
    volatile int8_t  tmpChar;
    
    if(!pUart)
    {
      #ifdef DEBUG_UART1
        NVIC_DisableIRQ(UART1_RX_TX_IRQn);
      #else
        NVIC_DisableIRQ(UART2_RX_TX_IRQn);
      #endif
      return;
    }
    
    stat = pUart->S1;
    numInts++;

    // If "framing error" or "overrun" error occours 
    // perform 'S1' cleanup. If not, 'S1' cleanup will be 
    // performed during regular reading of 'D' register.
    if (stat & (UART_S1_OR_MASK | UART_S1_FE_MASK))
    {
        // Reading register 'D' to cleanup 'S1' may cause 'RFIFO' underflow
        volatile uint8_t dummyByte = pUart->D;

        // If 'RFIFO' underflow detected, perform flush to reinitialize 'RFIFO'
        if (pUart->SFIFO & UART_SFIFO_RXUF_MASK)
        {
            pUart->CFIFO |= UART_CFIFO_RXFLUSH_MASK;
            pUart->SFIFO |= UART_SFIFO_RXUF_MASK;
        }

        // Nothing to process
        return;
    }

    // RX interrupt?
    if (stat & UART_S1_RDRF_MASK)
    {
        // reading 'D' register performs also cleanup of 'S1'
        tmpChar = pUart->D;
        if (_CHARQ_NOT_FULL(&rxQueue)) 
        {
            _CHARQ_ENQUEUE(&rxQueue,tmpChar);
        }
        else
        {
            // Drpping...
            numRxDropped++;
        }
        numRxInts++;
    }

    // TX interrupt
    if (stat & UART_S1_TDRE_MASK)
    {
        // More to send?
        if (!_CHARQ_EMPTY(&txQueue))
        {
            _CHARQ_DEQUEUE(&txQueue, tmpChar);
            pUart->D = tmpChar;
        }
        else
        {
            /* All data sent, disable transmit interrupt */
            pUart->C2&= ~UART_C2_TIE_MASK;
            txStarted = 0;
        }
        numTxInts++;
    }
}


void UART1_ERR_IRQHandler (void)
{
}

static void setBaudrate(int baudRate)
{
    uint32_t moduleClock = SystemCoreClock; 
    uint32_t baudDivisor = 0;
    uint32_t brfa = 0;
    uint8_t abdh = 0;
    uint8_t abdl = 0;
	
		if(pUart == UART2)
			moduleClock = SystemCoreClock/2;

    // Sanity check and calculation
    if ((moduleClock > 0) && (baudRate > 0))
    {
        baudDivisor = moduleClock / (baudRate << 4); // 16 samples per bit ==> divide by 16 times baud rate
        if (baudDivisor > 0)
        {
            brfa = ((((moduleClock - baudRate*(baudDivisor << 4)) << 1) + (baudRate >> 1)) / baudRate);
            if (brfa >= 32) {
                baudDivisor += 1;
                brfa = 0;
            }
        }
        if (baudDivisor > ((UART_BDH_SBR_MASK << 8) | UART_BDL_SBR_MASK))
        {
            // Error, do nothing
            return;
        }
    }

    // Update registers
    abdh = (uint8_t)((baudDivisor >> 8) & UART_BDH_SBR_MASK);
    abdl = (uint8_t)(baudDivisor & UART_BDL_SBR_MASK);
		
    pUart->BDH = abdh;
    pUart->BDL = abdl;
    
    pUart->C4 &= (~ UART_C4_BRFA_MASK);
    pUart->C4 |= UART_C4_BRFA(brfa);

    return;
}

/*-----------------------------------------------------------------------------
 *       SER_Init:  Initialize Serial Interface
 *----------------------------------------------------------------------------*/
void SER_Init (int baudRate) {
#ifdef __DBG_ITM
  ITM_RxBuffer = ITM_RXBUFFER_EMPTY;

#else

  /* Init rx and tx queues */
  _CHARQ_INIT(&rxQueue);
  _CHARQ_INIT(&txQueue);

  /* Configure UART1 as ALT3 function on pins PCT14 and PTC14                 */
  /* Settings: 115200 baud @ 100MHz, 8 data bits, 1 stop bit, no flow control  */

	#ifdef DEBUG_UART1
		pUart = UART1;
	#else
		pUart = UART2;
	#endif

  SIM->SCGC5 |= SIM_SCGC5_PORTE_MASK;   /* Enable PORTE gate clocking         */ 
	if(UART1 == pUart)
	{
		SIM->SCGC4 |= SIM_SCGC4_UART1_MASK;   										/* Enable UART 1 gate clocking        */
		PORTE->PCR[0] = 0 | PORT_PCR_MUX(3) | PORT_PCR_DSE_MASK; 	/* Pin mux configured as ALT3, RX     */
		PORTE->PCR[1] = 0 | PORT_PCR_MUX(3) | PORT_PCR_DSE_MASK; 	/* Pin mux configured as ALT3, TX     */
		PORTE->PCR[3] = 0 | PORT_PCR_MUX(3) | PORT_PCR_DSE_MASK; 	/* Pin mux configured as ALT3, TX     */
	}
	else if(UART2 == pUart)
	{
		SIM->SCGC4 |= SIM_SCGC4_UART2_MASK;   										/* Enable UART 1 gate clocking        */
		PORTE->PCR[17] = 0 | PORT_PCR_MUX(3) | PORT_PCR_DSE_MASK; /* Pin mux configured as ALT3, RX     */
		PORTE->PCR[16] = 0 | PORT_PCR_MUX(3) | PORT_PCR_DSE_MASK; /* Pin mux configured as ALT3, TX     */
	}
  /* Make sure RE and TE are disabled */
  pUart->C2 &= ~(UART_C2_RE_MASK | UART_C2_TE_MASK);

  // Set baud rate
  setBaudrate(baudRate);

  /* 8-bit mode. Normal operation */
  pUart->C1 = 0;

  /* Disable wakeups */
  pUart->C2 &= (~ (UART_C2_RWU_MASK));
  pUart->C4 &= (~ (UART_C4_MAEN1_MASK | UART_C4_MAEN2_MASK));
  pUart->MA1 = 0;
  pUart->MA2 = 0;

  /* Disable all error interrupts */
  pUart->C3 = 0;

  /* Configure watermark for RX */
  pUart->RWFIFO = UART_RWFIFO_RXWATER(1);

  /* Set watermark in the almost full TX buffer */
  if (((pUart->PFIFO & UART_PFIFO_TXFIFOSIZE_MASK) >> UART_PFIFO_TXFIFOSIZE_SHIFT) == 0)
  {
     /* 1 dataword in D */
     pUart->TWFIFO = UART_TWFIFO_TXWATER(0);
  }
  else
  {
     uint8_t txsize = 1 << (((pUart->PFIFO & UART_PFIFO_TXFIFOSIZE_MASK) >> UART_PFIFO_TXFIFOSIZE_SHIFT) + 1);

     /* Watermark for TX buffer generates interrupts below & equal to watermark */
     pUart->TWFIFO = UART_TWFIFO_TXWATER(txsize - 1);
  }

  //pUart->TWFIFO = UART_TWFIFO_TXWATER(0);
	//pUart->TWFIFO = 1;

  /* Enable TX FIFO, enable RX FIFO */
  pUart->PFIFO |= UART_PFIFO_TXFE_MASK | UART_PFIFO_RXFE_MASK;

  /* Flush RX / TX buffers */
  pUart->CFIFO |= UART_CFIFO_RXFLUSH_MASK | UART_CFIFO_TXFLUSH_MASK;
	if(pUart == UART1)
	{
		/* enable hardware support for 485 RTS pin drive */
		pUart->MODEM &= (~(UART_MODEM_RXRTSE_MASK | UART_MODEM_TXCTSE_MASK));
		pUart->MODEM |= (UART_MODEM_TXRTSE_MASK | UART_MODEM_TXRTSPOL_MASK);
	}

  /* Transmitter and receiver enable */
  pUart->C2 = UART_C2_RIE_MASK  |       /* RDRF interrupt or DMA transfer requests enabled.   */
              UART_C2_RE_MASK   |       /* Receiver enable                    */
              UART_C2_TE_MASK;          /* Transmitter enable                 */

  /* Enable UART interrupts*/
  setIrq(1);
#endif
}


/*-----------------------------------------------------------------------------
 *       SER_PutChar:  Write a character to the Serial Port
 *----------------------------------------------------------------------------*/
int32_t SER_PutChar (int32_t ch) {
#ifdef __DBG_ITM
  int i;
  ITM_SendChar (ch & 0xFF);
  for (i = 10000; i; i--);
#else
  
  /* Disable interrupt */
  setIrq(0);

  /* Add character to queue */
  if (_CHARQ_FULL(&txQueue))
  {
      /* Drop it! */
      numTxDropped++;

      /* Done*/
      setIrq(1);
      return (ch & 0xFF);
  }
  else
  {
      _CHARQ_ENQUEUE(&txQueue, ch);
  }

  /* Tx started? */
  if (!txStarted)
  {
      /* Enable transmit interrupt */
      txStarted = 1;
      pUart->C2 |= UART_C2_TIE_MASK;
  }

  /* Done */
  setIrq(1);
  return (ch & 0xFF);

#endif 
}


/*-----------------------------------------------------------------------------
 *       SER_GetChar:  Read a character from the Serial Port
 *----------------------------------------------------------------------------*/
int32_t SER_GetChar (void) {
#ifdef __DBG_ITM
  if (ITM_CheckChar())
    return ITM_ReceiveChar();
#else
  
  int32_t tmpChar = (-1);

  /* Disable interrupt */
  setIrq(0);

  /* Character ready? */
  if (_CHARQ_SIZE(&rxQueue) > 0)
  {
      _CHARQ_DEQUEUE(&rxQueue, tmpChar);
  }

  /* Done */
  setIrq(1);
  return tmpChar;
#endif
}

int setIrq(int state)
{
	if(!pUart)
		return 0;
	if(state)
	{
		if(pUart == UART1)
			NVIC_EnableIRQ(UART1_RX_TX_IRQn);
		else if(pUart == UART2)
			NVIC_EnableIRQ(UART2_RX_TX_IRQn);
	}
	else
	{
		if(pUart == UART1)
			NVIC_DisableIRQ(UART1_RX_TX_IRQn);
		else if(pUart == UART2)
			NVIC_DisableIRQ(UART2_RX_TX_IRQn);
	}
		
	return 1;
}

/*-----------------------------------------------------------------------------
 * End of file
 *----------------------------------------------------------------------------*/
